DA2             bit     P2.7
DA1             bit     P2.6
DA0             bit     P2.5

CS1             bit     P2.4
CS0             bit     P2.3

ATA_nDIOW       bit     P3.4        ;Write Pulse
ATA_nDIOR       bit     P3.5        ;Read Pulse
ATA_INTRQ       bit     P3.6        ;interrupt Request
ATA_nRESET      bit     P3.7

ATA_DATA_MS     equ     P0           ;MSB D8-D15
ATA_DATA_LS     equ     P1           ;LSB D0-D7

ATA_ADDR        equ     P2
ATAPI_LOW       equ     0
ATAPI_HIGH      equ     0ffh


;--------------------------------------
do_packet_cmd:
                mov     r0,#packet
                mov     r7,#16
clr_packet:
                mov     @r0,#0
                inc     r0
                djnz    r7,clr_packet

                lcall   ndrq_wait

                mov     a,#0a0h
                lcall   set_cmd_register
                ret
;--------------------------------------
send_packet:
                lcall   drq_wait

                mov     r0,#packet

                mov     r7,packet_size          ;packet_size
send_packet_1:
                mov     a,@r0
                inc     r0
                push    acc
                mov     a,@r0
                inc     r0
                mov     b,a
                pop     acc

                lcall   set_data
                djnz    r7,send_packet_1

                lcall   wait_irq
                lcall   get_status_register
                ret
;--------------------------------------
get_MSF_field:
                lcall   get_data
                mov     tmp1,b
;
                lcall   get_data
                mov     tmp2,a
                mov     tmp3,b
                ret
;--------------------------------------
packet_cmd_error:
                mov     dptr,#scl_packet_error
                lcall   write_line
;
                lcall   get_error_register
                ret
;--------------------------------------
init_ata_device:

                clr     ATA_nRESET
                lcall   Atapi_Delaytime
                setb    ATA_nRESET
                lcall   Atapi_Delaytime
;
                lcall   bsy_wait
                lcall   get_devhead_register
                clr     ATA_DATA_LS.4           ;set device are "master"
                lcall   set_devhead_register
;
                lcall   bsy_wait
                lcall   get_cyl_low_register
                cjne    a,#14h,init_device_error
                lcall   get_cyl_high_register
                cjne    a,#0ebh,init_device_error
                ljmp    init_device_done
init_device_error:
                mov     dptr,#scl_device_error
                lcall   write_line
                ajmp    $
init_device_done:                               ;diagnostics Command
                mov     a,#90h
                lcall   set_cmd_register
                lcall   bsy_wait

                lcall   get_error_register
                jnb     acc.0,init_device_error
;After init successfull
;Display CD-ROM Ready
                mov     dptr,#scl_device_ready
                lcall   write_line
                mov     r2,#5
                lcall   dtsec
                ret
;--------------------------------------
atapi_identify_device:

                mov     a,#2
                lcall   set_cyl_high_register
                clr     a
                lcall   set_cyl_low_register

                mov     a,#0a1h                 ;Command
                lcall   set_cmd_register
                lcall   wait_irq

                lcall   get_data
                clr     a
                jb      acc.0,identify_C2
                jb      acc.1,identify_C2
                mov     a,#6
identify_C2:
                jb      acc.0,identify_C3
                jb      acc.1,identify_C3
                mov     a,#8

identify_C3:
                mov     packet_size,a
;skip next 9 word

                mov     r7,#26
                lcall   skip_device_fields
;get model number
                mov     a,#LINE2
                lcall   goto_xy

                mov     r7,#10
                lcall   print_device_ansi       ;get Company Vendor

                mov     r7,#(10+33+1+1+46+1)
                lcall   skip_device_fields

                lcall   skip_rest_of_packet

                lcall   bsy_drdy_wait

                mov     dptr,#scl_device_ready
                lcall   write_line
                mov     r2,#5
                lcall   dtsec
                ret
;--------------------------------------
test_unit_ready:
                lcall   do_packet_cmd
                lcall   send_packet
                jnb     STATUS_REG_ERR,test_unit_d
                clr     c
                ret
test_unit_d:
                setb    c
                ret
;--------------------------------------
wait_for_ready:
                mov     dptr,#scl_chk_drive
                lcall   write_line
;
                mov     r3,#2           ;5
wait_for_ready2:
                mov     r2,#15          ;25
wait_for_ready1:

                dec     r2
                cjne    r2,#0,xx
                ajmp    not_media
xx:
                lcall   Atapi_Delaytime
                lcall   test_unit_ready
                jnc     wait_for_ready1
                dec     r3
                cjne    r3,#0,wait_for_ready2
media_insert:
                lcall   read_TOC
                setb    flag_disc
                ret
not_media:
                mov     dptr,#scl_nodisc
                lcall   write_line
                clr     flag_disc
                ret
;--------------------------------------
read_TOC:
                mov     dptr,#scl_toc_reading
                lcall   write_line
                mov     dptr,#scl_waitting
                lcall   write_line
get_TOC:
                mov     a,#0ffh
                lcall   set_cyl_low_register
                lcall   set_cyl_high_register

                lcall   do_packet_cmd

                mov     packet_cmd,#43h         ;command
                mov     packet_1,#00000010b     ;MSF = 1
                mov     packet_7,#0ffh          ;allocation Lenght
                mov     packet_8,#0ffh          ;(again, set it to full ON)

                lcall   send_packet

                jb      STATUS_REG_ERR,read_toc_e
                jb      STATUS_REG_DRQ,read_toc_get_file
                setb    c
                ret
read_toc_e:
                mov     dptr,#scl_toc_error
                lcall   write_line
                lcall   delay_display
                clr     c
                ret
;--------------------------------------
read_toc_get_file:
                lcall   get_data        ;data lenght

                lcall   get_data        ;get_track Limit
                mov     start_track,a
;read_track_description
;
                mov     r1,#TOC_file+3  ;for zero
                mov     total_track,#0  ;clear
read_toc12:
                lcall   get_data        ;get address and Control Fields

                lcall   get_data        ;get Track number
                mov     tmp0,a
next_MSF_read:
;get MSF feilds
                lcall   get_data
                mov     tmp1,b          ;M
                mov     @r1,b
                inc     r1
;
                lcall   get_data
                mov     tmp2,a          ;S
                mov     @r1,a
                inc     r1
                mov     tmp3,b          ;F
                mov     @r1,b
                inc     r1
;
                inc     total_track
                lcall   bsy_wait
                jb      STATUS_REG_DRQ,read_toc12
                setb    c

;Find total Track
                dec     total_track     ;total_track = total_track - 1
;
                mov     dptr,#scl_TOCinfor
                lcall   copy_write
;show total track
                mov     xbyte_l,total_track
                lcall   insert_Word

                mov     disbuff+0,a     ;MSB Bit
                mov     a,r3
                mov     disbuff+1,a     ;LSB Bit
;Calculate Remaing Time

                mov     a,total_track
                inc     a               ;find for last
                mov     dptr,#TOC_file
                mov     b,#3
                mul     ab
                add     a,dpl
                mov     r0,a
                mov     a,@r0
                mov     min,a

                inc     r0
                mov     a,@r0
                mov     sec,a

                clr     flag_subb
                mov     a,total_track
                cjne    a,#1,none_equ
                setb    flag_subb
none_equ:
                mov     a,min

                jb      flag_subb,ad1
                clr     c
                subb    a,TOC_file+3    ;M_Start
ad1:
                mov     xbyte_l,a
                lcall   insert_Word     ;Total Min
;
                mov     disbuff+9,a
                mov     a,r3
                mov     disbuff+10,a
;
                mov     a,sec
                jb      flag_subb,ad2
                clr     c
                subb    a,TOC_file+4    ;S_Start
ad2:
                mov     xbyte_l,a
                lcall   insert_Word     ;Total Sec
                cjne    a,#'0',Wr_Char
                mov     a,#' '
Wr_Char:
                mov     disbuff+13,a
                mov     a,r3
                mov     disbuff+14,a
                lcall   write_disbuf
                ret
;--------------------------------------
calculate_track:
;start play track
                mov     a,track_play    ;start index track
                mov     r1,#Start_M
                lcall   load_MSF
;end of play
                mov     a,total_track   ;stop end of TOC
                inc     a               ;add last MSF
                mov     r1,#End_M
                lcall   load_MSF
                ret
;--------------------------------------
Load_MSF:
                mov     dptr,#TOC_file
                mov     b,#3
                mul     ab
                add     a,dpl
                mov     r0,a

                mov     b,#3
Load_MSF_file:
                clr     a
                mov     a,@r0
                mov     @r1,a

                inc     r0
                inc     r1
                djnz    b,load_MSF_file
                ret

;--------------------------------------
play_CD:
                mov     dptr,#scl_playCD
                lcall   write_line
;
Seek_track:
                lcall   calculate_track

;                lcall   sc_test

                lcall   do_packet_cmd
                mov     packet_cmd,#47h
                mov     packet_3,Start_M
                mov     packet_4,Start_S
                mov     packet_5,Start_F
;
                mov     packet_6,End_M
                mov     packet_7,End_S
                mov     packet_8,End_F

                lcall   send_packet
                jnb     STATUS_REG_ERR,play_audio_cmd
                lcall   packet_cmd_error
                clr     c
                clr     flag_play
                ret
play_audio_cmd:
                setb    flag_play
                setb    c
                ret
;--------------------------------------
read_subchanCmd:
                mov     a,#0ffh
                lcall   set_cyl_low_register
                lcall   set_cyl_high_register
;
                lcall   do_packet_cmd
                mov     packet_cmd,#42h
                mov     packet_1,#00000010b     ;MSF = 1
                mov     packet_2,#01000000b     ;subQ = 1
                mov     packet_3,#01h
                mov     packet_7,#0ffh          ;allocation lenght
                mov     packet_8,#0ffh          ;
;
                lcall   send_packet
                jb      STATUS_REG_ERR,read_subcmdErr
                jb      STATUS_REG_DRQ,read_subcmdRdy
                ret
read_subcmdErr:
;                lcall   packet_cmd_error
                lcall   wait_for_ready
                clr     flag_play
                ret
;--------------
read_subcmdRdy:
                lcall   get_data        ;get audio status
                mov     a,b
                mov     audio_status,a  ;wait for Stop

;                lcall   print_serial
;                lcall   lf

                lcall   get_data        ;sub chanel data lenght

                lcall   get_data        ;format code ADDR

                lcall   get_data        ;track inDex
                mov     index_play,a    ;track <now> play
                mov     track_play,a    ;store for seek




                lcall   get_MSF_field   ;show inDex Remaing Time
;
                lcall   get_MSF_field

                jb      flag_repeat,r_repeat
                mov     dptr,#scl_Play_Time
                ajmp    ltrack
r_repeat:
                mov     dptr,#scl_Play_repeat
ltrack:
                lcall   copy_write

;show track
                mov     xbyte_l,index_play
                lcall   insert_Word
                                        ;Index Track
                mov     disbuff+6,a     ;MSB Bit
                mov     a,r3
                mov     disbuff+7,a     ;LSB Bit
;Min Display
                mov     xbyte_l,tmp1    ;Min
                lcall   insert_Word
                cjne    a,#'0',Wr_CharM
                mov     a,#' '
Wr_CharM:
                mov     disbuff+9,a     ;MSB Bit
                mov     a,r3
                mov     disbuff+10,a     ;LSB Bit
;Sec Display
                mov     xbyte_l,tmp2    ;Sec
                lcall   insert_Word

                mov     disbuff+13,a     ;MSB Bit
                mov     a,r3
                mov     disbuff+14,a     ;LSB Bit
;
                lcall   write_disbuf
                acall   skip_rest_of_packet
                mov     r2,#2
                lcall   dtsec
                ret
;--------------------------------------
atapi_rew_track:
                jnb     flag_play,Reword_Exit
                mov     a,track_play
                xrl     a,#1
                jnz     Reword_Track
Reword_Exit:
                ret
Reword_track:
                clr     c
                mov     a,track_play
                subb    a,#1
                mov     track_play,a
                ljmp    Seek_Track
;--------------------------------------
atapi_few_track:
                jnb     flag_play,Forword_Exit

                mov     a,total_track
                xrl     a,track_play
                jnz     Forword_track
Forword_Exit:
                ret
Forword_track:
                mov     a,track_play
                add     a,#1
                mov     track_play,a
                ljmp    Seek_Track
;--------------------------------------
atapi_pause:
                cpl     flag_pause
                jnb     flag_pause,none_pause

                mov     dptr,#scl_pause
                lcall   write_line
none_pause:
                lcall   do_packet_cmd
                mov     packet_cmd,#4bh
                clr     a
                mov     c,flag_pause
                cpl     c
                mov     acc.0,c
                mov     packet_8,a              ;pause / Resume

                lcall   send_packet
                jnb     STATUS_REG_ERR,Pause_Cmd
                lcall   packet_cmd_error
                ret
Pause_Cmd:
                ret
;--------------------------------------
;stop Function
atapi_stop:

                mov     dptr,#scl_stopCD
                lcall   write_line

                lcall   do_packet_cmd            ;clear
                mov     packet_cmd,#4eh
                lcall   send_packet
                jnb     STATUS_REG_ERR,Stop_playCmd
                lcall   packet_cmd_error
                ljmp    stop_exit

Stop_playCmd:
                lcall   wait_for_ready  ;check disc and get TOC Disc
                                        ;after stop CD read New TOC

                clr     flag_play
stop_exit:
                ret
;--------------------------------------
atapi_eject:

                mov     dptr,#scl_stopCD
                lcall   write_line

                lcall   do_packet_cmd   ;clear
                mov     packet_cmd,#1bh         ;command
                jb      flag_eject,Load_tray

                mov     dptr,#scl_tray_open
                lcall   write_line

                mov     packet_4,#00000010b
                lcall   send_packet
                cpl     flag_eject
                ret
Load_tray:
                mov     dptr,#scl_tray_close
                lcall   write_line

                mov     packet_4,#00000011b
                lcall   send_packet
                cpl     flag_eject
                lcall   wait_for_ready  ;check disc and get TOC Disc
                ret
;--------------------------------------
skip_rest_of_packet:
                lcall   get_status_register
                jnb     STATUS_REG_DRQ,skip_rest_d
                lcall   get_data
                sjmp    skip_rest_of_packet
skip_rest_d:
                ret
;--------------------------------------
send_device_ansi:
                lcall   get_data
                push    acc
                mov     a,b
                lcall   tx_byte
                pop     acc
                lcall   tx_byte
                djnz    r7,send_device_ansi
                ret

;--------------------------------------
print_device_ansi:
                lcall   get_data
                push    acc
                mov     a,b
                lcall   write_lcd
                pop     acc
                lcall   write_lcd
                djnz    r7,print_device_ansi

                mov     r2,#5
                lcall   dtsec
                ret
;----------------------------------
STATUS_REG_ERR   bit    acc.0
STATUS_REG_IDX   bit    acc.1
STATUS_REG_CORR  bit    acc.2
STATUS_REG_DRQ   bit    acc.3
STATUS_REG_DSC   bit    acc.4
STATUS_REG_DF    bit    acc.5
STATUS_REG_DRDY  bit    acc.6
STATUS_REG_BSY   bit    acc.7

get_status_register:
;set status

                setb    DA2
                setb    DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
get_sector_n_register:
                clr     DA2
                setb    DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
get_sector_c_register:
                clr     DA2
                setb    DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
ERROR_REG_AMNF  bit     acc.0
ERROR_REG_TKONF bit     acc.1
ERROR_REG_ABRT  bit     acc.2
ERROR_REG_MCR   bit     acc.3
ERROR_REG_IDNF  bit     acc.4
ERROR_REG_MC    bit     acc.5
ERROR_REG_UNC   bit     acc.6
ERROR_REG_RES   bit     acc.7

get_error_register:
                clr     DA2
                clr     DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
get_cyl_high_register:
                setb    DA2
                clr     DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
get_cyl_low_register:
                setb    DA2
                clr     DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
get_devhead_register:
                setb    DA2
                setb    DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle_1s
                ret
;--------------------------------------
set_devhead_register:
                setb    DA2
                setb    DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle_1s
                ret
;--------------------------------------
set_cyl_high_register:
                setb    DA2
                clr     DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle_1s
                ret
;--------------------------------------
set_cyl_low_register:
                setb    DA2
                clr     DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle_1s
                ret
;--------------------------------------
set_cmd_register:
                setb    DA2
                setb    DA1
                setb    DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle_1s
                ret
;--------------------------------------
;data Register
get_data:
                clr     DA2
                clr     DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   read_cycle
                ret
;--------------------------------------
set_data:
                clr     DA2
                clr     DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle
                ret
;--------------------------------------
Set_Data_1s:
                clr     DA2
                clr     DA1
                clr     DA0
                setb    CS1
                clr     CS0
                lcall   write_cycle_1s
                ret
;--------------------------------------
Read_cycle:
                clr     ATA_nDIOR
                mov     a,ATA_DATA_LS
                mov     b,ATA_DATA_MS
                setb    ATA_nDIOR
                ret
;--------------------------------------
read_cycle_1s:
                mov     ATA_DATA_LS,#0ffh
                clr     ATA_nDIOR
                mov     a,ATA_DATA_LS
                setb    ATA_nDIOR
                ret
;--------------------------------------
write_cycle:
                mov     ATA_DATA_LS,a
                mov     ATA_DATA_MS,b
                clr     ATA_nDIOW
                nop
                setb    ATA_nDIOW
                ret
;--------------------------------------
write_cycle_1s:
                mov     ATA_DATA_LS,a
                clr     ATA_nDIOW
                nop
                setb    ATA_nDIOW
                ret
;--------------------------------------
Wait_Irq:
                setb    ATA_INTRQ
                jnb     ATA_INTRQ,Wait_Irq
                ret
;--------------------------------------
skip_device_fields:
                lcall   get_data
                djnz    r7,skip_device_fields
                ret
;--------------------------------------
bsy_wait:
                nop
                nop
                nop
                nop
                lcall   get_status_register
                jb      STATUS_REG_BSY,bsy_wait
                ret
;--------------------------------------
bsy_drdy_wait:
                nop
                nop
                nop
                nop
                lcall   Atapi_Delaytime
                lcall   get_status_register
                jb      STATUS_REG_BSY,bsy_drdy_wait
                jnb     STATUS_REG_DRDY,bsy_drdy_wait
                ret
;--------------------------------------
drq_wait:
                nop
                nop
                nop
                nop
                lcall   get_status_register
                jb      STATUS_REG_BSY,drq_wait
                jnb     STATUS_REG_DRQ,drq_wait
                ret
;--------------------------------------
ndrq_wait:
                nop
                nop
                nop
                nop
                lcall   get_status_register
                jb      STATUS_REG_BSY,drq_wait
                jb      STATUS_REG_DRQ,drq_wait
                ret
;--------------------------------------
Atapi_Delaytime:
                mov     r4,#0ah
delay_1:        mov     r5,#0ah
delay_2:        mov     r6,#0
delay_3:        xch     a,r7
                xch     a,r7
                djnz    r6,delay_3
                djnz    r5,delay_2
                djnz    r4,delay_1
                ret
;----------------------------------------
delay_display:
                mov     r2,#3
                lcall   dtsec
                ret
;----------------------------------------
convert_hd:
                mov     xbyte_h,#0
                lcall   htod16bit
                mov     a,mem_buff+2
                lcall   htoa
                ret
;---------------------------------------
insert_Word:
                lcall   convert_hd
                mov     a,r2
                ret
;----------------------------------------

                                        ;0123456701234567
scl_device_error:       db      LINE2,  "ATAPI CD-ROM ERR"
scl_device_ready:       db      LINE1,  "ATAPI CD-ROM Rdy"

scl_chk_drive:          db      LINE1,  "Disc insert Wait"
scl_nodisc:             db      LINE2,  " Tray No Disc.! "

scl_toc_reading:        db      LINE1,  "  TOC Reading.. "
scl_waitting:           db      LINE2,  " Please Wait...."
scl_toc_error:          db      LINE2,  " Read TOC Error!"

scl_description:        db      LINE1,  "Read M.S.F Files"

scl_tray_open:          db      LINE2,  "-< Tray Open >- ";
scl_tray_close:         db      LINE2,  "-> Tray Close<- ";

scl_playCD:             db      LINE1,  "Now Playing     ";
scl_packet_error:       db      LINE1,  "Cmd Packet Error";

scl_TOCinfor:           db              "   Track 00m 00s";
scl_Play_Time:          db              "Play  01 00m 00s";
scl_Play_repeat:        db              "Rep-D 01 00m 00s";
scl_Track_repeat:       db              "Rep-T 01 00m 00s";


scl_stopCd:             db      LINE1,  "Now Stop..      ";
scl_empty:              db      LINE2,  "                ";

scl_pause:              db      LINE2,  "Now CD Pause..! ";
